﻿using FabApprovalWorkerService.Models;

using System.Text;

namespace FabApprovalWorkerService.Services;

public interface ITrainingService {
    Task<IEnumerable<int>> GetTrainingIdsForECN(int ecnNumber);
    Task MarkTrainingAsComplete(int trainingId);
    Task DeleteTrainingAssignment(int trainingId);
    Task<IEnumerable<int>> GetTrainingAssignmentIdsForTraining(int trainingId);
    Task DeleteDocAssignment(int trainingAssignmentId);
    Task<IEnumerable<TrainingAssignment>> GetActiveTrainingAssignments();
    Task UpdateTrainingAssignmentLastNotification(int trainingAssignmentId);
    Task<int> GetEcnNumberByTrainingId(int trainingId);
}

public class TrainingService : ITrainingService {
    private ILogger<TrainingService> _logger;
    private IDalService _dalService;

    public TrainingService(ILogger<TrainingService> logger, IDalService dalService) {
        _logger = logger ??
            throw new ArgumentNullException("ILogger not injected");
        _dalService = dalService ??
            throw new ArgumentNullException("IDalService not injected");
    }

    public async Task DeleteDocAssignment(int trainingAssignmentId) {
        if (trainingAssignmentId <= 0) throw new ArgumentException($"Invalid training assignment id: {trainingAssignmentId}");
        
        try {
            _logger.LogInformation($"Attempting to delete training doc assignments for training assignment {trainingAssignmentId}");

            StringBuilder queryBuilder = new();
            queryBuilder.Append($"update TrainingDocAcks set Deleted = 1, ");
            queryBuilder.Append($"DeletedDate = '{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}' ");
            queryBuilder.Append($"where TrainingAssignmentID = {trainingAssignmentId} and Reviewed = 0;");

            await _dalService.ExecuteAsync(queryBuilder.ToString());
        } catch (Exception ex) {
            StringBuilder errMsgBuilder = new();
            errMsgBuilder.Append($"An exception occurred when attempting to delete training doc assignment ");
            errMsgBuilder.Append($"{trainingAssignmentId}. Exception: {ex.Message}");
            _logger.LogError(errMsgBuilder.ToString());
            throw;
        }
    }

    public async Task DeleteTrainingAssignment(int trainingId) {
        if (trainingId <= 0) throw new ArgumentException($"Invalid training id: {trainingId}");

        try {
            _logger.LogInformation($"Attempting to delete training assignment {trainingId}");

            StringBuilder queryBuilder = new();
            queryBuilder.Append($"update TrainingAssignments set Deleted = 1, ");
            queryBuilder.Append($"DeletedDate = '{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}' ");
            queryBuilder.Append($"where TrainingID = {trainingId} and status = 0;");

            await _dalService.ExecuteAsync(queryBuilder.ToString());
        } catch (Exception ex) {
            StringBuilder errMsgBuilder = new();
            errMsgBuilder.Append($"An exception occurred when attempting to delete training assignment ");
            errMsgBuilder.Append($"{trainingId}. Exception: {ex.Message}");
            _logger.LogError(errMsgBuilder.ToString());
            throw;
        }
    }

    public async Task MarkTrainingAsComplete(int trainingId) {
        if (trainingId <= 0) throw new ArgumentException($"Invalid training id: {trainingId}");

        try {
            _logger.LogInformation($"Attempting to mark training {trainingId} as complete");

            StringBuilder queryBuilder = new();
            queryBuilder.Append($"update Training set Status = 1, ");
            queryBuilder.Append($"CompletedDate = '{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}' ");
            queryBuilder.Append($"where TrainingID = {trainingId};");

            await _dalService.ExecuteAsync(queryBuilder.ToString());
        } catch (Exception ex) {
            StringBuilder errMsgBuilder = new();
            errMsgBuilder.Append($"An exception occurred when attempting to mark training {trainingId} ");
            errMsgBuilder.Append($"as complete. Exception: {ex.Message}");
            _logger.LogError(errMsgBuilder.ToString());
            throw;
        }
    }

    public async Task<IEnumerable<int>> GetTrainingAssignmentIdsForTraining(int trainingId) {
        if (trainingId <= 0) throw new ArgumentException($"Invalid trainingID: {trainingId}");

        try {
            _logger.LogInformation($"Attempting to get training assignment ids for training id {trainingId}");

            string sql = $"select ID from TrainingAssignments where TrainingID = {trainingId};";

            return await _dalService.QueryAsync<int>(sql);
        } catch (Exception ex) {
            StringBuilder errMsgBuilder = new();
            errMsgBuilder.Append($"An exception occurred when attempting to get training assignment ids ");
            errMsgBuilder.Append($"for training id {trainingId}. Exception: {ex.Message}");
            _logger.LogError(errMsgBuilder.ToString());
            throw;
        }
    }

    public async Task<IEnumerable<int>> GetTrainingIdsForECN(int ecnNumber) {
        if (ecnNumber <= 0) throw new ArgumentException($"Invalid ecnNumber: {ecnNumber}");

        try {
            _logger.LogInformation($"Attempting to get training ids for ecn {ecnNumber}");

            string sql = $"select TrainingID from Training where ECN = {ecnNumber};";

            return await _dalService.QueryAsync<int>(sql);
        } catch (Exception ex) {
            StringBuilder errMsgBuilder = new();
            errMsgBuilder.Append($"An exception occurred when attempting to get training ids ");
            errMsgBuilder.Append($"for ECN {ecnNumber}. Exception: {ex.Message}");
            _logger.LogError(errMsgBuilder.ToString());
            throw;
        }
    }

    public async Task<IEnumerable<TrainingAssignment>> GetActiveTrainingAssignments() {
        try {
            _logger.LogInformation($"Attempting to get active training assignments");

            StringBuilder queryBuilder = new();
            queryBuilder.Append("select ID, UserID, DateAssigned, TrainingID, status, Deleted, DeletedDate, LastNotification ");
            queryBuilder.Append("from TrainingAssignments where status = 0 and (Deleted is null or Deleted = 0);");

            return await _dalService.QueryAsync<TrainingAssignment>(queryBuilder.ToString());
        } catch (Exception ex) {
            StringBuilder errMsgBuilder = new();
            errMsgBuilder.Append($"An exception occurred when attempting to get active training assignments. ");
            errMsgBuilder.Append($"Exception: {ex.Message}");
            _logger.LogError(errMsgBuilder.ToString());
            throw;
        }
    }

    public async Task UpdateTrainingAssignmentLastNotification(int trainingAssignmentId) {
        try {
            _logger.LogInformation($"Attempting to update last notification date for training assignment {trainingAssignmentId}");

            if (trainingAssignmentId <= 0)
                throw new ArgumentException($"{trainingAssignmentId} is not a valid training assignment Id");

            StringBuilder queryBuilder = new();
            queryBuilder.Append($"update TrainingAssignments set LastNotification = {DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss.fff")}");
            queryBuilder.Append($"where ID = {trainingAssignmentId};");

            await _dalService.ExecuteAsync(queryBuilder.ToString());
        } catch (Exception ex) {
            StringBuilder errMsgBuilder = new();
            errMsgBuilder.Append("An exception occurred when attempting to update last notification ");
            errMsgBuilder.Append($"for training assignment {trainingAssignmentId}. Exception: {ex.Message}");
            _logger.LogError(errMsgBuilder.ToString());
            throw;
        }
    }

    public async Task<int> GetEcnNumberByTrainingId(int trainingId) {
        try {
            _logger.LogInformation($"Attempting to get ECN number for training {trainingId}");

            if (trainingId <= 0)
                throw new ArgumentException($"{trainingId} is not a valid training Id");

            string sql = $"select e.ECNNumber from Training t join ECN e on t.ECN = e.ECNNumber where t.TrainingID = {trainingId};";

            int ecnNumber = (await _dalService.QueryAsync<int>(sql)).FirstOrDefault();

            if (ecnNumber <= 0) throw new Exception($"ECN number not found for training {trainingId}");

            return ecnNumber;
        } catch (Exception ex) {
            StringBuilder errMsgBuilder = new();
            errMsgBuilder.Append($"An exception occurred when attempting to get ECN number for training {trainingId}. ");
            errMsgBuilder.Append($"Exception: {ex.Message}");
            _logger.LogError(errMsgBuilder.ToString());
            throw;
        }
    }
}
